# -*- coding: utf-8 -*-
# @Time    : 2023/4/18 23:00
# @Author  : hxq
# @Software: PyCharm
# @File    : main.py
from hxq.interface.db_helper import DBHelper
from hxq.interface.httpx import HTTP, http

# if __name__ == '__main__':
#     CONFIG = {
#         'SQL_CREATOR': 'MySQL',
#         'SQL_HOST': '127.0.0.1',
#         'SQL_USER': 'root',
#         'SQL_PASSWORD': '123456',
#         'SQL_DATABASE': 'blog'
#     }
#     db = DBHelper(config=CONFIG)
#     print(db.all("SHOW DATABASES;"))
#     print(db.first("SELECT * FROM  rule_group"))

if __name__ == '__main__':
    HTTP().download(
        'https://infinitypro-img.infinitynewtab.com/wallpaper/travel/5596.jpg?attname=infinity-19779309.jpg'
    )
